#-*-Python-*-#
"""
Suite of functions for creating and working with Spherical Embeddings

The functions implemented here are either extant in the literature or
have been developed specifically for this project by Shaun Ault and
Roy Joshua.  

AUTHORS:

- Shaun Ault (2010): Primary coder and developer.

- Roy Joshua: Began the project around 2006, and with the help
of Ault created a GAP version of the script, BJAlgorithm.g.  

EXAMPLES:

Create a spherical embedding over A2 with weight polytope generated
by the simple root (1,0)::

    sage: sph = SphericalEmbedding("A2")
    sage: sph.setpolytope( [ (1,0) ] )
    [[1, 0], [-1, 0], [-1, -1], [0, -1], [1, 1], [0, 1]]

Find the moment polytope of the previous::

    sage: M = sph.moment_polytope()
    sage: M.vertices()
    [[0, 0], [1, 1/2], [1/2, 1], [1, 1]]

"""

#*****************************************************************************
#       Copyright (C) 2011 Shaun Ault <ault@fordham.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  http://www.gnu.org/licenses/
#*****************************************************************************

class SphericalEmbedding(SageObject):
    """
    Initiate a Spherical Embedding object.  This is the starting
    place for any further computations.

    INPUT:
    
    - ``group`` - String representing a reductive group, such as
    "A3", "B3xD4", etc.
    
    - ``torus`` - [optional] Dimension of maximal torus.
    
    - ``polytope`` - [optional - can be set later] Weight polytope.
    
    OUTPUT:
    
    - ``sph`` - SphericalEmbedding object.
    
    EXAMPLES::
    
        sage: sph = SphericalEmbedding("A2xD4", torus=1)
        sage: sph
        Spherical Embedding: Group = A2xD4xT1; Weight polytope = None
    
    """
    def __init__(self, group=None, **kwds):
        """
        See ``SphericalEmbedding`` for full documentation.

        """
        # Group structure
        torus = kwds.get('torus', 0)
        if group != None:
            rs = RootSystem(group)
            roots = rs.ambient_space().roots()
            simple = rs.ambient_space().simple_roots()
            cartan = cartan_matrix(group)  ## Transpose of LiE Cartan matrix
            ss_rank = cartan.nrows()

            # Fundamental weights
            fundamental = rs.ambient_space().fundamental_weights()
            F = matrix(QQ, [vector(row) for row in fundamental])

            # Euclidean inner product matrix and lattice transformation
            inprodmat = copy(cartan).change_ring(QQ)
            for i in range(ss_rank):
                c = (vector(simple[i+1]) * vector(simple[i+1])) / 2
                inprodmat.rescale_row(i, c)
                F.rescale_row(i, 1/c)
            lattice_trans = F.transpose()
            weight_trans = inprodmat * F

            # Weyl group
            # In Sage, the Weyl group must be of simple type, so we
            #  create a list of component Weyl groups
            ctype = CartanType(group)
            Weyl_group = []
            if hasattr(ctype, 'component_types'):
                comp = ctype.component_types()
                cur = 0
                W_words = [ [] ]
                length = 0
                for c in comp:
                    Wc = WeylGroup(c)
                    Weyl_group.append(copy(Wc))
                    cur += Wc.ngens()
            else:
                W = WeylGroup(ctype)
                Weyl_group.append(copy(W))
                
            # Set class variables
            self._group = group
            self._torus = torus
            self._roots = roots
            self._simple_system = simple
            self._lattice_transform = lattice_trans
            self._fundamental_weights = fundamental
            self._cartan = cartan.block_sum(identity_matrix(torus))
            self._weight_transform = weight_trans
            self._inprodmat = inprodmat.block_sum(identity_matrix(torus))
            self._ss_rank = ss_rank
            self._rank = ss_rank + torus
            self._Weyl_group = Weyl_group
            self._IP_X_tree = []
        else:
            # group is a torus, T^n
            self._group = None
            self._torus = torus
            self._roots = []
            self._inprodmat = identity_matrix(QQ, torus)
            self._ss_rank = 0
            self._rank = torus
            self._fundamental_weights = None
            self._IP_X_tree = []

        # Weight polytope
        w_polytope = kwds.get('polytope', None)
        if w_polytope != None:
            self.setpolytope(w_polytope)

    def _repr_(self):
        name = "Spherical Embedding: "
        group = self._group
        if group == None:
            G = ""
        else:
            G = group
        polytope = self.weight_polytope()
        if polytope == None:
            poly = "None"
        else:
            poly = str(polytope.vertices())
        if self._torus > 0:
            if group == None:
                tor = "T" + str(self._torus)
            else:
                tor = "xT" + str(self._torus)
        else:
            tor = ""
        return name+"Group = "+G+tor+"; Weight polytope = "+poly

    ## "Set" methods
    def setpolytope(self, vertices):
        """
        INPUT:

        -  ``vertices`` - vertices in the root lattice

        OUTPUT: 

        - ``polytope`` - the convex hull of the Weyl-group translates
        of the vertices
        """
        if hasattr(self, '_moment_polytope'):
            del self._moment_polytope
        if hasattr(self, '_Newton_polytope'):
            del self._Newton_polytope
        self._IP_X_tree = []

        new_vertices = []
        W_words = self.Weyl_words()
        for v in vertices:
            for s in W_words:
                new_vertices.append(self.Weyl_action(v, s))
        
        self._weight_polytope = Polyhedron(vertices=new_vertices)
        return self._weight_polytope.vertices()
        
    ## "Get" methods
    def group(self):
        """
        Name of the semisimple group.

        EXAMPLES::
    
            sage: sph = SphericalEmbedding("A3xB2", torus=1)
            sage: sph
            Spherical Embedding: Group = A3xB2xT1; Weight polytope = None
            sage: sph.group()
            'A3xB2'
        """
        return self._group

    def toral_dim(self):
        """
        Dimension of torus.  (Not the maximal torus.)

        EXAMPLES::
    
            sage: sph = SphericalEmbedding("A3xB2", torus=1)
            sage: sph.toral_dim()
            1
        """
        return self._torus

    def rank(self):
        """
        Rank of the Lie group

        EXAMPLES::
    
            sage: sph = SphericalEmbedding("A3xB2", torus=1)
            sage: sph.rank()
            6
        """
        return self._rank

    def semisimple_rank(self):
        """
        Semisimple rank of the Lie group

        EXAMPLES::
    
            sage: sph = SphericalEmbedding("A3xB2", torus=1)
            sage: sph.semisimple_rank()
            5
        """
        return self._ss_rank

    def simple_system(self, i):
        """
        The i^{th} root in a simple root system.

        EXAMPLES::
    
            sage: sph = SphericalEmbedding("A3xB2", torus=1)
            sage: sph.simple_system(1)
            (1, -1, 0, 0, 0, 0)
        """
        return self._simple_system[i]

    def fundamental_system(self):
        """
        The fundamental weights, as a Sage finite family object.

        EXAMPLES::
    
            sage: sph = SphericalEmbedding("A3xB2", torus=1)
            sage: sph.fundamental_system()
            Finite family {1: (1, 0, 0, 0, 0, 0), 2: (1, 1, 0, 0, 0, 0), 3: (1, 1, 1, 0, 0, 0), 4: (0, 0, 0, 0, 1, 0), 5: (0, 0, 0, 0, 1/2, 1/2)}
        """
        return self._fundamental_weights

    def weight_polytope(self):
        """
        The weight polytope.

        EXAMPLES::
    
            sage: sph = SphericalEmbedding("A2", polytope=[(1,0)])
            sage: sph.weight_polytope()
            A 2-dimensional polyhedron in QQ^2 defined as the convex hull of 6 vertices.
        """
        if hasattr(self, '_weight_polytope'):
            return self._weight_polytope
        else:
            return None

    def Weyl_words(self):
        """
        The Weyl group, as a list of Weyl words.  Explicit Weyl words are
        computed only when the fuction is first called.

        EXAMPLES::
        
            sage: sph = SphericalEmbedding("A2")
            sage: sph.Weyl_words()
            [[], [1], [1, 2], [1, 2, 1], [2], [2, 1]]

        """
        if self._group == None:
            return [ [] ]
        if not hasattr(self, '_Weyl_words'):
            Weyl_group = self._Weyl_group
            cur = 0
            W_words = [ [] ]
            for Wc in Weyl_group:
                Wc_words = [[i+cur for i in g.reduced_word()] for g in Wc]
                W_words = [flatten([w1, w2]) for w1 in W_words
                            for w2 in Wc_words]
                cur += Wc.ngens()
            self._Weyl_words = copy(W_words)
        return self._Weyl_words

    def Weyl_subgroup(self, rho):
        """
        INPUT:

        -  ``rho`` - a list of Weyl group generators, as integers
        
        OUTPUT: 

        - ``W`` - the list of Weyl words generated by rho
        
        EXAMPLES::

            sage: sph = SphericalEmbedding("A2")
            sage: sph.Weyl_subgroup([1])
            [[], [1]]
        """
        W = []
        for w in self.Weyl_words():
            if len(set(w).difference(set(rho))) == 0:
                W.append(w)
        return W

    def Weyl_minimal_coset_reps(self, sigma, rho):
        """
        INPUT:
        
        - ``sigma`` - a list of Weyl group generators
        
        - ``rho`` - a subset of sigma

        OUTPUT:

        - ``WQ`` - the list of minimal coset representatives of
        W_sigma/W_rho, where W_sigma and W_rho are subgroups of
        the Weyl group, generated by the roots in sigma and rho,
        respectively.

        EXAMPLES::

            sage: sph = SphericalEmbedding("A2")
            sage: sph.Weyl_minimal_coset_reps([1,2],[1])
            [[], [1, 2], [2]]
        """
        W_sigma = self.Weyl_subgroup(sigma)
        W_rho = self.Weyl_subgroup(rho)
        n_cosets = len(W_sigma)/len(W_rho)
        cosets = [ copy(W_rho) ]
        for g in W_sigma:
            flag = False
            # Determine if g is in current cosets
            for coset in cosets:
                for h in coset:
                    if h == g:
                        flag = True
                        break
                if flag:
                    break
            if not flag:
                # Create a new coset
                new_coset = [self.reduce_Weyl_word(flatten([g,h])) 
                             for h in W_rho]
                cosets.append(copy(new_coset))
                if len(cosets) == n_cosets:
                    break

        # Find minimal coset representatives
        WQ = []
        for coset in cosets:
            minimal = coset[0]
            for i in range(1, len(coset)):
                if len(coset[i]) < len(minimal):
                    minimal = coset[i]
            WQ.append(copy(minimal))
        return WQ

    def reduce_Weyl_word(self, s):
        """
        INPUT:
        
        - ``s`` - a Weyl group element, as a Weyl word

        OUTPUT:

        - ``s0`` - The reduced Weyl word corresponding to s.
        """
        # Parse s into components
        N = [c.ngens() for c in self._Weyl_group]
        n = len(N)
        for i in range(1, n):
            N[i] += N[i-1]
        comp = [ [] for i in N ]
        for i in s:
            for j in range(n):
                if i <= N[j]:
                    if j > 0:
                        i -= N[j-1]
                    comp[j].append(i)
                    break

        # Reduce each component
        new_comp = []
        for j in range(n):
            g = comp[j]
            Wc = self._Weyl_group[j]
            new_comp.append(Wc.from_reduced_word(g).reduced_word())

        # Build composite Weyl word
        s0 = new_comp[0]
        for j in range(1, n):
            s0.extend( [i+N[j-1] for i in new_comp[j] ] )

        return s0

    def n_generated_roots(self, rho):
        """
        INPUT:
        
        - ``rho`` - a list of simple roots

        OUTPUT:

        - ``n_roots`` - The number of roots in the span of rho

        EXAMPLES::
        
            sage: sph = SphericalEmbedding("A3")
            sage: sph.n_generated_roots([1,2])
            6
        """
        if rho==[]:
            return 0

        basis = [ vector(QQ, self.simple_system(i)) for i in rho ]
        V = span(basis, QQ)
        roots = []
        for alpha in self._roots:
            if vector(QQ, alpha) in V:
                roots.append(alpha)
        return len(roots)

    def Weyl_action(self, vec, s):
        """
        INPUT:

        -  ``vec`` - a vector in the root lattice, that is,
        vec = v_1*alpha_1 + ... + v_n*alpha_n

        -  ``s`` - a Weyl group element, as a Weyl word
        
        OUTPUT: 

        - ``vec_s`` - the left action of s on vec, as a vector
        in the root lattice.

        EXAMPLES::
        
            sage: sph = SphericalEmbedding("A2")
            sage: sph.Weyl_action( (1,3), [2,1] )
            (-3, -2)
        """
        v = copy(vector(vec))
        for i in s:
            v[i-1] -= v * self._cartan.row(i-1)
        return v

    def Lie_dim(self, d, rho):
        """
        INPUT:

        - ``d`` - The dimension of the weight polytope

        - ``rho`` - List of simple roots

        OUTPUT:

        - ``lie_dim`` - Dimension of the embedding.

        EXAMPLES::

            sage: sph = SphericalEmbedding("A2", polytope=[(1,0)])
            sage: sph.Lie_dim(2, [1,2])
            8
        """
        W = self.Weyl_subgroup(rho)
        length = max([len(g) for g in W])
        lie_dim = 2*length + d
        return lie_dim

    def GC_polytope(self, lam):
        """
        INPUT:

        - ``lam`` - a dominant weight

        OUTPUT:
        
        - ``polyhedron`` - the GelfandCetlin polytope

        EXAMPLES::

            sage: sph = SphericalEmbedding("A2", polytope=[(1,0)])
            sage: sph.GC_polytope( [2,1,0] ).vertices()
            [[1, 0, 1], [1, 0, 0], [1, 1, 1], [2, 0, 2], [2, 0, 0], [2, 1, 1], [2, 1, 2]]
        """
        ineq = []
        if self._group[0] == 'A':
            n = self._ss_rank + 1
            if len(lam) != n:
                raise ValueError("weight is not the correct length")
            m = (n^2 - n + 2)/2
            c = 1
            for i in range(n-1):
                for j in range(n-i-1):
                    row = [0 for N in range(m)]
                    row[c] = -1
                    if i == 0:
                        row[0] = lam[c-1]
                    else:
                        row[c+i-n] = 1
                    ineq.append(copy(row))

                    row = [0 for N in range(m)]
                    row[c] = 1

                    if i == 0:
                        row[0] = -lam[c]
                    else:
                        row[c+i-n+1] = -1
                        
                    ineq.append(copy(row))
                    c += 1

        elif self._group[0] == 'C':
            n = self._ss_rank
            if len(lam) != n:
                raise ValueError("weight is not the correct length")
            m = n^2 + 1
            c = 1
            for i in range(2*n-1):
                for j in range(n - floor((i+1)/2)):
                    row = [0 for N in range(m)]
                    row[c] = -1
                    if i == 0:
                        row[0] = lam[c-1]
                    else:
                        pass
                        row[c - n + floor(i/2)] = 1

                    ineq.append(copy(row))
                    row = [0 for N in range(m)]
                    row[c] = 1
                    if i == 0 and c < n:
                        row[0] = -lam[c]
                    else:
                        if ((i+1)/2).is_integral() or j+1 < n-floor((i+1)/2):
                            row[c - n + floor(i/2) + 1] = -1

                    ineq.append(copy(row))
                    c += 1
        else:
            raise NotImplementedError

        return Polyhedron(ieqs=ineq)

    def moment_polytope(self):
        """
        OUTPUT:
        
        - ``m_polytope`` - the moment polytope corresponding to
        the current weight polytope

        EXAMPLES::

            sage: sph = SphericalEmbedding("A2", polytope=[(1,0)])
            sage: sph.moment_polytope().vertices()
            [[0, 0], [1, 1/2], [1/2, 1], [1, 1]]
        """
        if not hasattr(self, '_weight_polytope'):        
            return "Please set weight polytope first"

        if hasattr(self, '_moment_polytope'):
            return self._moment_polytope

        zeros = matrix(QQ, self._ss_rank, 1)
        dominant = zeros.augment(self._inprodmat)
        pos_polytope = Polyhedron(ieqs=dominant.rows())
        m_polytope = self._weight_polytope.intersection(pos_polytope)
        self._moment_polytope = copy(m_polytope)

        return m_polytope

    def Newton_polytope(self):
        """
        OUTPUT:
        
        - ``N_polytope`` - the Newton polytope

        EXAMPLES::

            sage: sph = SphericalEmbedding("A2", polytope=[(1,0)])
            sage: sph.Newton_polytope().vertices()
            [[0, 0, 0, 0, 0, 0], [3/2, 0, 0, 0, 0, 0], [3/2, 0, 0, 3/2, 0, 0], [3/2, 0, 0, 3/2, 0, 3/2], [3/2, 3/2, 0, 3/2, 3/2, 3/2], [3/2, 3/2, 0, 3/2, 0, 3/2], [3/2, 3/2, 0, 3/2, 0, 0], [2, 1, 0, 1, 0, 1], [2, 1, 0, 1, 0, 0], [2, 1, 0, 1, 1, 1], [2, 1, 0, 2, 0, 2], [2, 1, 0, 2, 0, 0], [2, 1, 0, 2, 1, 1], [2, 1, 0, 2, 1, 2]]
        """
        if self._group[0] != 'A' and self._group[0] != 'C':
            raise NotImplementedError

        if not hasattr(self, '_weight_polytope'):
            return "Please set weight polytope first"

        if not hasattr(self, '_moment_polytope'):
            self.moment_polytope()

        if hasattr(self, '_Newton_polytope'):
            return self._Newton_polytope
        
        Phi = self._moment_polytope.vertices()
        weights = [root_vector(v, self).weight_vector() for v in Phi]
        vertices = []
        for lam in weights:
            Delta_lam = self.GC_polytope(lam).vertices()
            for v in Delta_lam:
                L = list(lam)
                L.extend(v)
                vertices.append(copy(L))

        N_polytope = Polyhedron(vertices=vertices)
        self._Newton_polytope = copy(N_polytope)
        
        return N_polytope

    def weight_volume(self, polytope):
        """
        INPUT:

        - ``polytope`` - a polytope in the root lattice

        OUTPUT:
        
        - ``volume`` - the volume, in terms of the weight lattice.
        Requires optional "lrs" package.

        EXAMPLES::
            sage: sph = SphericalEmbedding("A2", polytope=[(1,0)])
            sage: sph.weight_volume(sph.moment_polytope())
            1.5
        """
        V = polytope.vertices()
        P = Polyhedron([root_vector(v, self).weight_vector() for v in V])
        return P.lrs_volume()

    def admissible(self, face, rho):
        """
        INPUT:

        - ``face`` - a face of a polytope in the root lattice

        - ``rho`` - a list of simple roots

        OUTPUT:
        
        - ``admissible`` - returns true, if face is rho-admissible.

        EXAMPLES::
            
            sage: sph = SphericalEmbedding("A2", polytope=[(1,0)])
            sage: sph.admissible([[1,0], [1,1]], [1,2])
            True
            sage: sph.admissible([[-1,0]], [1,2])
            False
        """
        
        ## Quick check 1:
        ## Vacuous cases
        if len(face) == 0 or len(rho) == 0:
            return True
        
        rho1 = [i-1 for i in rho]
        rho_mat = self._inprodmat.matrix_from_columns(rho1)
        M = matrix(QQ, face) * rho_mat

        ## Quick check 2:
        ## Check whether any point is in positive rho-chamber
        for i in range(M.nrows()):
            admissible = True
            for j in range(M.ncols()):
                if M[i][j] <= 0:
                    admissible = False
        if admissible:
            return True
        #if len(face) == 1:
        #    return False

        ## Quick check 3:
        ## Check whether there are points within two distinct 
        ##  positive chamber walls
        walls = []
        for row in M:
            pos = True
            L = []
            for j in range(len(row)):
                if row[j] < 0:
                    pos = False
                if row[j] == 0:
                    L.append(j)

            if pos and len(L) > 0:
                walls.append(copy(L))

        for i in range(len(walls)):
            for j in range(i+1, len(walls)):
                if len(set(walls[i]).intersection(set(walls[j]))) == 0:
                    return True

        ## Final check:
        ## Build the intersection polytope
        face_polytope = Polyhedron(vertices=face)
        zeros = matrix(QQ, len(rho), 1)
        dominant = zeros.augment(rho_mat.transpose())
        pos_polytope = Polyhedron(ieqs=dominant.rows())
        intersection = face_polytope.intersection(pos_polytope)
        dim = intersection.dim()
        
        if dim < face_polytope.dim():
            return False
        else:
            return True

    def admissible_faces(self, polytope, rho):
        """
        INPUT:

        - ``polytope`` - a polytope in the root lattice

        - ``rho`` - a list of simple roots

        OUTPUT:
        
        - ``adm_faces`` - the faces of polytope that are rho-admissible
        
        EXAMPLES::
            
            sage: sph = SphericalEmbedding("A2", polytope=[(1,0)])
            sage: sph.admissible_faces(sph.weight_polytope(), [1,2])
            [[[1, 1]], [[1, 1], [0, 1]], [[1, 0], [1, 1]], [[1, 0], [-1, 0], [-1, -1], [0, -1], [1, 1], [0, 1]]]
        """
        vertices = polytope.vertices()
        F = polytope.face_lattice()
        faces = []
        for f in F:
            face = []
            vface = f.element.Vrepresentation()
            for v in vface:
                face.append(list(v.vector()))
            faces.append(face)
        faces.remove([])
        
        adm_faces = []
        for f in faces:
            if self.admissible(f, rho):
                adm_faces.append(copy(f))

        return adm_faces

    def stabilizing_roots(self, vertices, rho):
        """
        INPUT:

        - ``vertices`` - a list of vertices in the root lattice

        - ``rho`` - a list of simple roots

        OUTPUT:
        
        - ``roots`` - the subset of rho that fix vertices set-wise,
        though not necessarily point-wise

        EXAMPLES::
            
            sage: sph = SphericalEmbedding("A2", polytope=[(1,0)])
            sage: sph.stabilizing_roots([[1,1], [0,1]], [1,2])
            [1]
        """
        roots = []
        for lam in rho:
            add_root = True
            for v in vertices:
                new_v = self.Weyl_action(v, [lam])
                if not list(new_v) in vertices:
                    add_root = False
                    break
            if add_root:
                roots.append(lam)

        return roots

    def fixing_roots(self, vertices, rho):
        """
        INPUT:

        - ``vertices`` - a list of vertices in the root lattice

        - ``rho`` - a list of simple roots

        OUTPUT:
        
        - ``roots`` - the subset of rho that fix the vertices point-wise

        EXAMPLES::
            
            sage: sph = SphericalEmbedding("A2", polytope=[(1,0)])
            sage: sph.fixing_roots([[1,1], [0,1]], [1,2])
            []
        """
        roots = []
        for lam in rho:
            add_root = True
            for v in vertices:
                new_v = self.Weyl_action(v, [lam])
                if not list(new_v) == v:
                    add_root = False
                    break
            if add_root:
                roots.append(lam)
        return roots

    def normal_polytope(self, delta, phi, rho):
        """
        INPUT:

        - ``delta`` - polytope in the root lattice as a set of vertices

        - ``phi`` - a face of delta as a subset of delta.

        - ``rho`` - a list of simple roots

        OUTPUT:
        
        - ``np`` - a normal polytope to phi

        - ``rho_np`` - simple reflections in the corresponding group
          of np
        
        EXAMPLES::

            sage: sph = SphericalEmbedding("A2", polytope=[(2,1)])
            sage: delta = sph.weight_polytope().vertices()
            sage: phi = [[2,1]]
            sage: sph.normal_polytope(delta, phi, [1,2])
            [[(0, -1), (0, 1)], [2]]

        COMMENTS:
        - No check is done to ensure phi is a subset of delta
        
        """        
        ## Normal Cone
        pv = [vector(QQ, v) for v in phi]
        dv = [vector(QQ, v) for v in delta]

        ## c_vectors is the list of vectors from the basepoint to
        ##  points not in phi.
        ## pv[0] is the chosen basepoint.
        c_vectors = []
        for v in dv:
            if not v in pv:
                c_vectors.append(v - pv[0])

        ## phi_vectors is the list of vectors from the basepoint
        ##  to points in phi.
        phi_vectors = []
        for i in range(1, len(pv)):
            phi_vectors.append(pv[i] - pv[0])
        if len(pv) > 1:
            ## Orthogonal complement of space spanned by phi
            M = self._inprodmat * matrix(QQ, phi_vectors).transpose()
            N = [root_vector(v, self) for v in M.left_kernel().basis()]
            
            ## Normalize the basis vectors
            for j in range(len(N)):
                for i in range(j):
                    N[j] -= ((N[j] * N[i])/(N[i] * N[i]))*N[i]

            ## Project c_vectors onto orthogonal complement
            nc_vectors = []
            for v in c_vectors:
                rv = root_vector(v, self)
                new_v = root_vector(0, self)
                for w in N:
                    new_v += ((rv * w)/(w * w))*w
                nc_vectors.append(new_v.vector())
        else:
            # phi is a single point.  All c_vectors are orthogonal
            nc_vectors = copy(c_vectors)

        ## Fixing roots of phi, which are the roots for the sub Weyl group,
        ##   W_{\phi}
        fix_phi = self.fixing_roots(phi, rho)

        ## Normal polytope
        if len(nc_vectors)==0:
            raise NotImplementedError
        # First we find a hyperplane that intersects the normal
        #   cone non-trivially.
        M = matrix(QQ, nc_vectors)
        if M.rank()==1:
            # If M has rank 1, then the normal vector is very easy to find!
            normal = vector(QQ, M.row(0))
        else:
            V_mat = M * self._inprodmat
            m = V_mat.nrows()
            n = V_mat.ncols()
            # Find a normal to such a hyperplane by finding a solution to
            #   the simultaneous inequalities, N \cdot v > 0.
            ieqs = matrix(QQ, m+2*n, n+1)
            for row in range(m):
                for col in range(n):
                    ieqs[row, col+1] = V_mat[row, col]
            # To get a reasonable answer, bound the components of N by 1 and -1.
            for row in range(m, m+n):
                for col in range(n+1):
                    if col==0:
                        ieqs[row, col] = 1
                    else:
                        ieqs[row, col] = kronecker_delta(row-m, col-1)
            for row in range(m+n, m+2*n):
                for col in range(n+1):
                    if col==0:
                        ieqs[row, col] = 1
                    else:
                        ieqs[row, col] = -kronecker_delta(row-m-n, col-1)
            H_normals = Polyhedron(ieqs = list(ieqs)).vertices()
            # Find a normal that is strictly in the interior of the bounds.
            normal = sum(vector(P) for P in H_normals)/len(H_normals)
        
        # Make the normal W_{\phi}-invariant
        if len(fix_phi) > 0:
            W_phi = self.Weyl_subgroup(fix_phi)
            normal_inv = 0 * normal
            for g in W_phi:
                normal_inv += self.Weyl_action(normal, g)
            normal = normal_inv

        # Scale normal to integral coordinates
        n = lcm([denominator(x) for x in normal])
        d = gcd([numerator(x) for x in normal])
        normal *= n/d

        # Find the points of the normal polytope
        np = []
        normal_rv = root_vector(normal, self)
        n = normal_rv * normal_rv
        for v in nc_vectors:
            np.append((n / (root_vector(v, self)*normal_rv))*v - normal)

        # Return the normal polytope and associated roots
        return [np, fix_phi]

    def _IP_X_recursive(self, delta, rho, t, parent):
        """
        INPUT:

        - ``delta`` - polytope in the root lattice

        - ``rho`` - a list of simple roots

        - ``t`` - the indeterminate for the polynomial
        
        - ``parent`` - index of parent node for the tree

        OUTPUT:
        
        - ``IP_X`` - polynomial

        See `IP_X` for examples.

        """
        IP_X = 0
        r = delta.dim()
        d = self.Lie_dim(r, rho)
        faces = self.admissible_faces(delta, rho)
        n_roots = self.n_generated_roots(rho)
        for phi in faces:
            # Node for this face
            node = dict()

            # Stabilizing and fixing roots
            I_phi = self.stabilizing_roots(phi, rho)
            J_phi = self.fixing_roots(phi, rho)
            K_phi = list(set(I_phi).difference(set(J_phi)))

            # Dimension, rank, etc.
            if len(phi)==1:
                d_phi = 0
            else:
                phi_poly = Polyhedron(phi)
                d_phi = phi_poly.dim()
            d_x = d - d_phi - n_roots + self.n_generated_roots(J_phi)
            r_x = 2 * r - d_phi

            # Poincare polynomials
            p_I = 0
            for g in self.Weyl_minimal_coset_reps(rho, I_phi):
                p_I += t^(2*len(g))
            p_K = 0
            for g in self.Weyl_subgroup(K_phi):
                p_K += t^(2 * len(g))
            P_Gx = (-1)^r_x * (t^2 - 1)^d_phi * p_I^2 * p_K

            # Recursive Step 
            if d_x >= 2:
                # Get the normal polytope
                np_data = self.normal_polytope(delta.vertices(), phi, rho)
                np = Polyhedron(np_data[0])
                rho_x = np_data[1]

                # Recursive call to IPX_x
                par = copy(phi)
                IP_PS = self._IP_X_recursive(np, rho_x, t, par)
                IP_X_x = ((1 - t^2)*IP_PS).taylor(t,0,d_x-1)
            else:
                IP_X_x = 1

            IP_X_x = t^(2*d_x) * IP_X_x.subs(t=1/t)

            # Create node of the tree
            node['parent'] = parent
            node['phi'] = copy(phi)
            node['I'] = I_phi
            node['J'] = J_phi
            node['K'] = K_phi
            node['dim'] = d_phi
            node['d_x'] = d_x
            node['r_x'] = r_x
            node['p_I'] = p_I.expand()
            node['p_K'] = p_K.expand()
            node['P_Gx'] = P_Gx.expand()
            if d_x >= 2:
                node['np'] = {'polytope': np.vertices(), 'roots': rho_x}
            node['IP_X_x'] = IP_X_x
            self._IP_X_tree.append(node)

            ## Add to IP_X the component polynomial corresponding to 
            ##  this admissible face.
            IP_X += P_Gx * IP_X_x
        return IP_X

    def IP_X(self):
        """
        OUTPUT:

        - ``poly`` - The intesection cohomology Poincare series

        EXAMPLES::

            sage: tetra = [(-1,0,0),(1,0,0),(0,1,0),(0,0,1)]
            sage: sphT = SphericalEmbedding(torus=3, polytope=tetra)
            sage: sphT.IP_X()
            t^6 + t^4 + t^2 + 1

            sage: sph = SphericalEmbedding("A2", polytope=[(1,0)])
            sage: sph.IP_X()
            t^16 + 2*t^14 + 4*t^12 + 7*t^10 + 8*t^8 + 7*t^6 + 4*t^4 + 2*t^2 + 1
        """
        self._IP_X_tree = []
        t = var('t')
        delta=self.weight_polytope()
        rho = [i+1 for i in range(self._ss_rank)]
        poly = self._IP_X_recursive(delta, rho, t, None)
        self._IP_X_tree.reverse()
        return poly.expand()

    def IP_X_tree(self):
        """
        OUTPUT:

        - ``tree`` - The tree generated as IP_X is computed.

        EXAMPLES::

            sage: sph = SphericalEmbedding("A2", polytope=[(2,1)])
            sage: sph.IP_X()
            t^16 + t^14 + t^12 + t^10 + t^8 + t^6 + t^4 + t^2 + 1

            sage: T = sph.IP_X_tree()
            sage: len(T)
            5
        """
        return self._IP_X_tree


class root_vector(SageObject):
    def __init__(self, vec, sph): 
        self._amb_sph_emb = sph
        if vec == 0:
            self._vector = zero(QQ^sph.rank())
        else:
            self._vector = vector(QQ, list(vec))

    def _repr_(self):
        vec = str(self._vector)
        group = str(self._amb_sph_emb.group())
        return vec + " in root lattice over " + group

    ## Operator Overloading
    def _verify(self, other):
        if not isinstance(other, root_vector):
            raise ValueError("Both summands must be root vectors")
        if not self._amb_sph_emb == other._amb_sph_emb:
            raise ValueError("Vectors must lie in the same root lattice")
        
    def __add__(self, other):
        self._verify(other)
        return root_vector(self._vector + other._vector, self._amb_sph_emb)
    def __sub__(self, other):
        self._verify(other)
        return root_vector(self._vector - other._vector, self._amb_sph_emb)
    def __mul__(self, other):
        """
        COMMENTS:
        -  Overloads "*" with the inner product function.

        -  For the groups B_n, the value of vec1 * vec2 will be one-half
        of the inner product as determined by LiE.
        """
        if isinstance(other, Integer) or isinstance(other, Rational):
            return self.__rmul__(other)
        self._verify(other)
        vec1 = self._vector
        vec2 = other._vector
        return vec1 * self._amb_sph_emb._inprodmat * vec2
    def __rmul__(self, other):
        return root_vector(other * self._vector, self._amb_sph_emb)
    def __neg__(self):
        return (-1)*self
    def __div__(self, other):
        if isinstance(other, Integer) or isinstance(other, Rational):
            return root_vector(self._vector / other, self._amb_sph_emb)
        raise ValueError("Scalar division by rational numbers only")

    # Functions
    def vector(self):
        return self._vector
    def ambient_spherical_embedding(self):
        return self._amb_sph_emb
    def weight_vector(self):
        sph = self._amb_sph_emb
        vec = self._vector
        return vec * sph._weight_transform
    def Weyl_action(self, g):
        sph = self._amb_sph_emb
        vec = self._vector
        vec2 = sph.Weyl_action(vec, g)
        return root_vector(vec2, sph)
    def representing_vector(self):
        sph = self._amb_sph_emb
        vec = self._vector
        vec2 = 0
        for i in range(sph.rank()):
            vec2 += vec[i] * sph.simple_system(i+1)
        return vec2

class mixed_volume_data(SageObject):
    def __init__(self, coeff, **kwds): 
        self._coeff = coeff
        typ = kwds.get('type', None)
        self._type = typ
    def _repr_(self):
        # String representation of data, as used by DEMICS
        coeff = self._coeff
        typ = self._type

        string = "\n# The dimension or the number of variables\n"
        string += "Dim = " + str(self.dim())
        string += "\n\n# The number of the distinct support sets\n"
        string += "Support = " + str(self.support())
        string += "\n\n# The number of elements in each support set\n"
        string += "Elem ="
        for i in self.support_sizes():
            string += " " + str(i)
        string += "\n\n# The multiplicity of each support set\n"
        string += "Type ="
        for i in self.type():
            string += " " + str(i)
        string += "\n\n\n"
        for i in range(self.support()):
            string += "# The elements of support set " + str(i+1) + "\n"
            for row in coeff[i]:
                for col in row:
                    string += str(col) + " "
                string += "\n"
            string += "\n"
        return string

    def setcoeff(self, coeff):
        self._coeff = coeff
    def settype(self, typ):
        self._type = typ

    def coeff(self):
        return self._coeff
    def dim(self):
        if not hasattr(self, '_dim'):
            dim = len(self._coeff[0][0])
            self._dim = dim
        return self._dim
    def support(self):
        if not hasattr(self, '_support'):
            support = len(self._coeff)
            self._support = support
        return self._support
    def support_sizes(self):
        coeff = self._coeff
        if not hasattr(self, '_support_sizes'):
            if type(coeff[0]) == list:
                sup_sizes = [len(c) for c in coeff]
            if (type(coeff[0]) == type(identity_matrix(QQ,1)) or
                type(coeff[0]) == type(identity_matrix(ZZ,1))):
                sup_sizes = [c.nrows() for c in coeff]
            self._support_sizes = sup_sizes
        return self._support_sizes    
    def type(self):
        if self._type == None:
            ## Default multiplicity type
            typ = [1 for i in range(self.support())]
            self._type = typ
        return self._type

    def mixed_volume(self):
        """
        OUTPUT:

        - ``vol`` - Mixed volume computation based on given data.

        EXAMPLES::

            sage: m1 = matrix(QQ, [[1,0,0,0],[0,1,0,0],[0,0,1,0],[0,0,0,1],[0,0,0,0],[1,1,1,1]])
            sage: m2 = 2*m1; m3 = 3*m1
            sage: data = mixed_volume_data( [m1, m2, m3], type=[2, 1, 1] )
            sage: data.mixed_volume()
            24
        """

        ## Write data to file
        string = self._repr_()
        filename = "demics_data"
        FILE = open(filename, "w")
        FILE.write(string)
        FILE.close()

        ## Send command to DEMICS
        outfile = "demics_output"
        command = "demics " + filename + " > " + outfile
        os.system(command)

        ## Get result
        FILE = open(outfile, "r")
        result = FILE.read()
        import re
        vol = Rational(re.findall('\d+', result)[1])
        return vol

################################################################################
## Test Cases
################################################################################

def test_all_IP_X():
    print "T1:"
    sphT1 = SphericalEmbedding(torus=1, polytope=[[0],[1]])
    print sphT1.weight_polytope().vertices()
    print sphT1.IP_X()

    print "T2:"
    sphT2 = SphericalEmbedding(torus=2, polytope=[(0,0),(1,0),(0,1)])
    print sphT2.weight_polytope().vertices()
    print sphT2.IP_X()

    tetra = [(0,0,0),(1,0,0),(0,1,0),(0,0,1)]
    sphT3 = SphericalEmbedding(torus=3, polytope=tetra)
    print "T3:"
    print sphT3.weight_polytope().vertices()
    print sphT3.IP_X()

    print "A1:"
    sphA1 = SphericalEmbedding("A1", polytope=[[1]])
    print sphA1.weight_polytope().vertices()
    print sphA1.IP_X()

    print "A1xT1:"
    sphA1T1 = SphericalEmbedding("A1", torus=1, polytope=[(1,0),(0,1)])
    print sphA1T1.weight_polytope().vertices()
    print sphA1T1.IP_X()    

    print "A2:"
    sphA2 = SphericalEmbedding("A2", polytope=[(1,0)])
    print sphA2.weight_polytope().vertices()
    print sphA2.IP_X()

    print "A2:"
    sphA2.setpolytope([(1,2)])
    print sphA2.weight_polytope().vertices()
    print sphA2.IP_X()

    print "A1xA1:"
    sphA1A1 = SphericalEmbedding("A1xA1", polytope=[(1,1)])
    print sphA1A1.weight_polytope().vertices()
    print sphA1A1.IP_X()

    print "A1xA1:"
    sphA1A1.setpolytope([(1,0), (0,1)])
    print sphA1A1.weight_polytope().vertices()
    print sphA1A1.IP_X()

    print "B2:"
    sphB2 = SphericalEmbedding("B2", polytope=[(2,1)])
    print sphB2.weight_polytope().vertices()
    print sphB2.IP_X()

    print "C3:"
    sphC3 = SphericalEmbedding("C3", polytope=[(5,3,1)])
    print sphC3.weight_polytope().vertices()
    print sphC3.IP_X()

    print "G2:"
    sphG2 = SphericalEmbedding("G2", polytope=[(1,2)])
    print sphG2.weight_polytope().vertices()
    print sphG2.IP_X()
